@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2005 John E. Davis
+Copyright (C) 2005, 2006 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -39,6 +39,7 @@ USA.
 #include "slsh.h"
 
 #include <signal.h>
+#include <errno.h>
 
 #ifndef USE_GNU_READLINE
 # define USE_GNU_READLINE 0
@@ -111,6 +112,14 @@ static void sig_sigtstp (int sig)
 }
 # endif
 
+# ifdef REAL_UNIX_SYSTEM
+/* This hook if a signal occurs while waiting for input. */
+static int getkey_intr_hook (void)
+{
+   return SLang_handle_interrupt ();
+}
+# endif
+
 static int TTY_Inited = 0;
 static void init_tty (void)
 {
@@ -135,6 +144,11 @@ static void init_tty (void)
 # endif
 	SLang_exit_error ("Error initializing terminal.");
      }
+
+# ifdef REAL_UNIX_SYSTEM
+   SLang_getkey_intr_hook = getkey_intr_hook;
+# endif
+
 # if SYSTEM_SUPPORTS_SIGNALS
    SLtty_set_suspend_state (1);
    SLsig_unblock_signals ();
@@ -187,29 +201,91 @@ static int open_readline ()
 #endif
 }
 
-static char *read_input_line (SLang_RLine_Info_Type *rline, char *prompt)
+#if USE_GNU_READLINE
+static void redisplay_dummy (void)
+{
+}
+#endif
+static char *read_input_line (SLang_RLine_Info_Type *rline, char *prompt, int noecho)
 {
    char *line;
+#ifdef REAL_UNIX_SYSTEM
+   int stdin_is_noecho = 0;
+#endif
 
    if (Use_Readline == 0)
      {
-	char buf[512];
-	
+	char buf[1024];
+	char *b;
+
 	fprintf (stdout, "%s", prompt); fflush (stdout);
-	if (NULL == fgets (buf, sizeof (buf), stdin))
+	if (noecho)
+	  {
+#ifdef REAL_UNIX_SYSTEM
+	     if (isatty (fileno(stdin)))
+	       {
+		  (void) SLsystem ("stty -echo");   /* yuk */
+		  stdin_is_noecho = 1;
+	       }
+#endif
+	  }
+	
+	line = buf;
+	while (1)
+	  {
+	     while (NULL == fgets (buf, sizeof (buf), stdin))
+	       {
+#ifdef EINTR
+		  if (errno == EINTR)
+		    {
+		       if (-1 == SLang_handle_interrupt ())
+			 {
+			    line = NULL;
+			    break;
+			 }
+		       continue;
+		    }
+#endif
+		  line = NULL;
+		  break;
+	       }
+	     break;
+	  }
+#ifdef REAL_UNIX_SYSTEM
+	if (stdin_is_noecho)
+	  (void) SLsystem ("stty echo");
+#endif
+	if (line == NULL)
 	  return NULL;
-	return SLmake_string (buf);
+
+	/* Remove the final newline */
+	b = line;
+	while (*b && (*b != '\n'))
+	  b++;
+	*b = 0;
+
+	return SLmake_string (line);
      }
 #if SYSTEM_SUPPORTS_SIGNALS
    init_tty ();
 #endif
 #if USE_GNU_READLINE
    (void) rline;
+   if (noecho == 0)
+     rl_redisplay_function = rl_redisplay;
+   else
+     {
+	/* FIXME: What is the proper way to implement this in GNU readline? */
+	(void) fputs (prompt, stdout); (void) fflush (stdout);
+	rl_redisplay_function = redisplay_dummy;
+     }
    line = readline (prompt);
+   rl_redisplay_function = rl_redisplay;
 #else
    SLtt_get_screen_size ();
    SLrline_set_display_width (rline, SLtt_Screen_Cols);
    Active_Rline_Info = rline;
+   (void) SLrline_set_echo (rline, (noecho == 0));
    line = SLrline_read_line (rline, prompt, NULL);
    Active_Rline_Info = NULL;
 #endif
@@ -265,7 +341,7 @@ static char *get_input_line (SLang_Load_Type *x)
      }
    else prompt = "       ";
 
-   line = read_input_line (Rline_Info, prompt);
+   line = read_input_line (Rline_Info, prompt, 0);
 
    if ((line == NULL) 
        && (parse_level == 0)
@@ -423,7 +499,7 @@ static void close_intrinsic_readline (void)
 }
 #endif
 	
-void slsh_readline_intrinsic (char *prompt)
+static int readline_intrinsic_internal (char *prompt, int noecho)
 {
    char *line;
 
@@ -433,13 +509,25 @@ void slsh_readline_intrinsic (char *prompt)
      {
 	Intrinsic_Rline_Info = SLrline_open (SLtt_Screen_Cols, SL_RLINE_BLINK_MATCH);
 	if (Intrinsic_Rline_Info == NULL)
-	  return;
+	  return -1;
 	(void) SLang_add_cleanup_function (close_intrinsic_readline);
      }
 #endif
    enable_keyboard_interrupt ();
 
-   line = read_input_line (Intrinsic_Rline_Info, prompt);
-   (void) save_input_line (Intrinsic_Rline_Info, line);
+   line = read_input_line (Intrinsic_Rline_Info, prompt, noecho);
+   if (noecho == 0)
+     (void) save_input_line (Intrinsic_Rline_Info, line);
    (void) SLang_push_malloced_string (line);
+   return 0;
+}
+
+void slsh_readline_intrinsic (char *prompt)
+{
+   (void) readline_intrinsic_internal (prompt, 0);
+}
+
+void slsh_readline_noecho_intrinsic (char *prompt)
+{
+   (void) readline_intrinsic_internal (prompt, 1);
 }
