@@ -1,6 +1,6 @@
 /* Array manipulation routines for S-Lang */
 /*
-Copyright (C) 2004, 2005 John E. Davis
+Copyright (C) 2004, 2005, 2006 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -228,7 +228,7 @@ int _pSLarray_next_index (SLindex_Type *dims, SLindex_Type *max_dims, unsigned i
 	num_dims--;
 
 	dims_i = dims [num_dims] + 1;
-	if (dims_i != (int) max_dims [num_dims])
+	if (dims_i < (int) max_dims [num_dims])
 	  {
 	     dims [num_dims] = dims_i;
 	     return 0;
@@ -859,17 +859,12 @@ convert_nasty_index_objs (SLang_Array_Type *at,
    total_num_elements = 1;
    for (i = 0; i < num_indices; i++)
      {
-	SLindex_Type max_index, min_index;
-	SLang_Object_Type *obj;
-	SLindex_Type at_dims_i;
-
-	at_dims_i = at->dims[i];
-	obj = index_objs + i;
+	SLang_Object_Type *obj = index_objs + i;
 	range_delta_buf [i] = 0;
 
 	if (obj->data_type == SLANG_ARRAY_INDEX_TYPE)
 	  {
-	     range_buf [i] = min_index = max_index = obj->v.index_val;
+	     range_buf [i] = obj->v.index_val;
 	     max_dims [i] = 1;
 	     index_data[i] = range_buf + i;
 	     is_dim_array[i] = 0;
@@ -877,7 +872,7 @@ convert_nasty_index_objs (SLang_Array_Type *at,
 #if SLANG_ARRAY_INDEX_TYPE != SLANG_INT_TYPE
 	else if (obj->data_type == SLANG_INT_TYPE)
 	  {
-	     range_buf [i] = min_index = max_index = obj->v.int_val;
+	     range_buf [i] = obj->v.int_val;
 	     max_dims [i] = 1;
 	     index_data[i] = range_buf + i;
 	     is_dim_array[i] = 0;
@@ -2600,10 +2595,6 @@ static int array_eqs_method (SLtype a_type, VOID_STAR ap, SLtype b_type, VOID_ST
    SLang_Array_Type *at, *bt, *ct;
    unsigned int i, num_dims, num_elements;
    SLang_Class_Type *a_cl, *b_cl, *c_cl;
-   int (*binary_fun) (int,
-		      SLtype, VOID_STAR, unsigned int,
-		      SLtype, VOID_STAR, unsigned int,
-		      VOID_STAR);
    int is_eqs;
    int *ip, *ipmax;
 
@@ -2654,7 +2645,7 @@ static int array_eqs_method (SLtype a_type, VOID_STAR ap, SLtype b_type, VOID_ST
    
    /* Do it the hard way */
 
-   if (NULL == (binary_fun = _pSLclass_get_binary_fun (SLANG_EQ, a_cl, b_cl, &c_cl, 0)))
+   if (NULL == _pSLclass_get_binary_fun (SLANG_EQ, a_cl, b_cl, &c_cl, 0))
      return 0;
 
    if (num_elements == 0)
@@ -2851,7 +2842,7 @@ static void array_where_last (void)
 {
    SLang_Array_Type *at;
    char *a_data;
-   SLindex_Type i, num_elements;
+   SLindex_Type i;
    SLindex_Type istart = 0;
    
    istart = -1;
@@ -2859,7 +2850,6 @@ static void array_where_last (void)
      return;
 
    a_data = (char *) at->data;
-   num_elements = (SLindex_Type) at->num_elements;
 
    i = istart + 1;
    while (i > 0)
