@@ -1,7 +1,7 @@
 /* -*- mode: C; mode: fold; -*- */
 /* string manipulation functions for S-Lang. */
 /*
-Copyright (C) 2004, 2005 John E. Davis
+Copyright (C) 2004, 2005, 2006 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -787,13 +787,11 @@ static void subbytes_cmd (SLstr_Type *a, int *n_ptr, int *m_ptr) /*{{{*/
    int m;
    unsigned int n;
    unsigned int lena;
-   char *amax;
 
    n = (unsigned int) (*n_ptr - 1);
    m = *m_ptr;
 
    lena = _pSLstring_bytelen (a);
-   amax = a + lena;
 
    if (n > lena) 
      n = lena;
@@ -811,7 +809,7 @@ static void substr_cmd (SLstr_Type *a, int *n_ptr, int *m_ptr) /*{{{*/
    int n, m;
    int lena;
    int ignore_combining;
-   char *a1, *amax;
+   char *a1;
    
    if (_pSLinterp_UTF8_Mode == 0)
      {
@@ -824,9 +822,6 @@ static void substr_cmd (SLstr_Type *a, int *n_ptr, int *m_ptr) /*{{{*/
    n = *n_ptr;
    m = *m_ptr;
 
-   lena = _pSLstring_bytelen (a);
-   amax = a + lena;
-
    lena = SLutf8_strlen ((SLuchar_Type *)a, ignore_combining);
 	
    if (n > lena) n = lena + 1;
@@ -1463,13 +1458,21 @@ static char *SLdo_sprintf (char *fmt) /*{{{*/
 	     break;
 
 	   case 'c':
+#if 0
 	     if (use_long)
+#endif
 	       {
 		  SLwchar_Type wc;
 		  SLuchar_Type utf8_buf[SLUTF8_MAX_MBLEN+1];
 
-		  if ((-1 == SLang_pop_wchar (&wc))
-		      || (NULL == (_pSLinterp_encode_wchar (wc, utf8_buf, &guess_size))))
+		  if (-1 == SLang_pop_wchar (&wc))
+		    return out;
+		  if ((_pSLinterp_UTF8_Mode == 0) && (wc <= 0xFF))
+		    {
+		       utf8_buf[0] = (unsigned char)wc;
+		       utf8_buf[1] = 0;
+		    }
+		  else if (NULL == SLutf8_encode_null_terminate (wc, utf8_buf))
 		    return out;
 		  ch = 's';
 		  str = (char *)utf8_buf;
