@@ -1,4 +1,5 @@
 #include <stdio.h>
+#include <string.h>
 #include <stdlib.h>
 #include <slang.h>
 #include <math.h>
@@ -137,13 +138,28 @@ static void fake_import (char *ns_name)
 int main (int argc, char **argv)
 {
    int i;
+   int utf8 = 0;
 
-   if (argc < 2)
+   for (i = 1; i < argc; i++)
      {
-	fprintf (stderr, "Usage: %s FILE...\n", argv[0]);
+	char *arg = argv[i];
+	if (*arg != '-')
+	  break;
+
+	if (0 == strcmp (arg, "-utf8"))
+	  {
+	     utf8 = 1;
+	     continue;
+	  }
+	i = argc;
+     }
+
+   if (i >= argc)
+     {
+	fprintf (stderr, "Usage: %s [-utf8] FILE...\n", argv[0]);
 	return 1;
      }
-   (void) SLutf8_enable (1);
+   (void) SLutf8_enable (utf8);
 
    if ((-1 == SLang_init_all ())
        || (-1 == SLang_init_array_extra ())
@@ -177,15 +193,16 @@ int main (int argc, char **argv)
    fpsetmask (~(FP_X_OFL|FP_X_INV|FP_X_DZ|FP_X_DNML|FP_X_UFL|FP_X_IMP));
 #endif
 
-   if (argc > 2)
+   if (i + 1 < argc)
      Ignore_Exit = 1;
 
-   for (i = 1; i < argc; i++)
+   while (i < argc)
      {
 	if (-1 == SLang_load_file (argv[i]))
 	  return 1;
+	i++;
      }
-   
+
    return SLang_get_error ();
 }
 
