@@ -377,6 +377,10 @@ static void dump_token (_pSLang_Token_Type *t)
 	b = ":";
 	break;
 	
+      case POW_TOKEN:
+	b = "^";
+	break;
+
       case ADD_TOKEN:
 	b = "+";
 	break;
